package space.kklochko.simple_jbdd.tests.factories;

import space.kklochko.simple_jbdd.annotations.Given;
import space.kklochko.simple_jbdd.annotations.Then;
import space.kklochko.simple_jbdd.annotations.Title;
import space.kklochko.simple_jbdd.annotations.When;
import space.kklochko.simple_jbdd.tests.Test;
import space.kklochko.simple_jbdd.tests.commands.AbstractTestCommand;
import space.kklochko.simple_jbdd.tests.commands.SimpleTestCommand;
import space.kklochko.simple_jbdd.tests.commands.decorators.BlockDecorator;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.Map;
import java.util.TreeMap;

public class TestCommandFactory {
    public AbstractTestCommand create(Test input) {
	    Class<?> aClass = input.getClass();
	    Map<String, Method> test_methods = this.getTestMethods(aClass);

        AbstractTestCommand<Test> aTestCommand = createTest(aClass, input);

        if(test_methods.containsKey("Then")) {
            Method then = test_methods.get("Then");
            String label = then.getAnnotation(Then.class).value();
            aTestCommand = new BlockDecorator<Test>(label, aTestCommand, then, "Then");
        }

        if(test_methods.containsKey("When")) {
            Method when = test_methods.get("When");
            String label = when.getAnnotation(When.class).value();
            aTestCommand = new BlockDecorator<Test>(label, aTestCommand, when, "When");
        }

        if(test_methods.containsKey("Given")) {
            Method when = test_methods.get("Given");
            String label = when.getAnnotation(Given.class).value();
            aTestCommand = new BlockDecorator<Test>(label, aTestCommand, when, "Given");
        }

        return aTestCommand;
    }

    protected AbstractTestCommand<Test> createTest(Class<?> aClass, Test input) {
        Title aTitle = aClass.getAnnotation(Title.class);
        String title = aClass.getName();

        if(aTitle != null)
            title = aTitle.value();

        return new SimpleTestCommand<Test>(title, input);
    }

    protected <T extends Test> Map<String, Method> getTestMethods(Class<?> aClass) {
        Map<String, Method> test_methods = new TreeMap<String, Method>();

        for(Method method : aClass.getDeclaredMethods()) {
            Annotation[] annotations = method.getAnnotations();

            if (annotations.length == 0)
                continue;

            if(method.isAnnotationPresent(Given.class))
                test_methods.put("Given", method);

            if(method.isAnnotationPresent(When.class))
                test_methods.put("When", method);

            if(method.isAnnotationPresent(Then.class))
                test_methods.put("Then", method);
        }

        return test_methods;
    }
}
